﻿using System;
using System.IO;
using System.Xml;
using System.Data;

namespace SPWSUtilities
{
    public class FileDownloader
    {
        public string TargetDir { get; set; }
        public bool Recursive { get; set; }
        public bool Overwrite { get; set; }

        public ServiceWrapper Wrapper { get; set; }

        /// <summary>
        /// For the list (or folder) given, download all items.
        /// </summary>
        public void DownloadListItems(string listName)
        {
            if (listName.Contains("/"))
            {
                //we're really talking about a folder (within a list).  Just break it up into list name/folder
                string[] listFolder = listName.Split(new char[] { '/' }, 2, StringSplitOptions.None);
                DownloadFolderItems(listFolder[0], listFolder[1]);
                return;
            }

            string targetDir = Path.Combine(this.TargetDir, listName);
            Console.WriteLine(string.Format("Web {0} List {1}", Wrapper.WebUrl, listName));

            WSLists.Lists service = Wrapper.ListsServiceInstance();

            //get List Items
            XmlNode xmlListItems = service.GetListItems(listName, null, null, null, null, null, string.Empty);
            XmlTextReader xmlTextReader = new XmlTextReader(xmlListItems.OuterXml, XmlNodeType.Element, null);
            DataSet dataSet = new DataSet();
            dataSet.ReadXml(xmlTextReader);

            if (dataSet.Tables["row"] != null)
            {
                foreach (DataRow row in dataSet.Tables["row"].Rows)
                {
                    FileRef fr = FileRef.Create(row, Wrapper.WebUrl);

                    if (fr.DocIcon.Length > 0)
                    {
                        //This is a file Link
                        DownloadItem(fr, targetDir);
                    }
                    else
                    {
                        //This is a folder
                        if (this.Recursive)
                            DownloadFolderItems(listName, fr.FileName);
                    }
                }
            }
        }

        /// <summary>
        /// download this one item to targetDir. 
        /// </summary>
        private void DownloadItem(FileRef fr, string targetDir)
        {
            if (!Directory.Exists(targetDir))
                Directory.CreateDirectory(targetDir);

            string destinationUrl = Path.Combine(targetDir, fr.FileName);

            //Receive Document Contents into byte array 
            byte[] fileContents = new Byte[4096];
            WSCopy.FieldInformation[] fieldInfoArray;
            Wrapper.CopyServiceInstance.GetItem(fr.FullUrl, out fieldInfoArray, out fileContents);

            if (!this.Overwrite && File.Exists(destinationUrl))
            {
                Console.WriteLine("File {0} already exists.  Use switch o to overwrite.", destinationUrl);
                return;
            }

            Console.WriteLine(string.Format("File {0}", destinationUrl));

            //Create a new file and write contents to that document
            using (FileStream fStream = new FileStream(destinationUrl, FileMode.Create, FileAccess.ReadWrite))
            {
                fStream.Write(fileContents, 0, fileContents.Length);
                fStream.Close();
            }
        }

        #region private methods etc.
        private void DownloadFolderItems(string listName, string docLibName)
        {
            //determine the local destination directory
            string targetDir = Path.Combine(this.TargetDir, listName);
            targetDir = Path.Combine(targetDir, docLibName);

            //instantiate the list service
            WSLists.Lists service = Wrapper.ListsServiceInstance();

            //Preapare XML Query to request the contents of a single folder. 
            XmlDocument xmlDoc = new System.Xml.XmlDocument();
            XmlNode ndQueryOptions = xmlDoc.CreateNode(XmlNodeType.Element, "QueryOptions", "");
            string folderPartialPath = listName + "/" + docLibName;
            if (folderPartialPath.StartsWith("/")) folderPartialPath = folderPartialPath.Substring(1);  //strip leading '/'
            ndQueryOptions.InnerXml = "<Folder>" + folderPartialPath + "</Folder>";

            XmlNode xmlListItems = service.GetListItems(listName, null, null, null, null, ndQueryOptions, null);
            XmlTextReader xmlTextReader = new XmlTextReader(xmlListItems.OuterXml, XmlNodeType.Element, null);
            DataSet dataSet = new DataSet();
            dataSet.ReadXml(xmlTextReader);

            if (dataSet.Tables["row"] != null)
            {
                foreach (DataRow row in dataSet.Tables["row"].Rows)
                {
                    FileRef fr = FileRef.Create(row, Wrapper.WebUrl);

                    if (fr.DocIcon.Length > 0)
                    {
                        //This is a file Link
                        DownloadItem(fr, targetDir);
                    }
                    else
                    {
                        //This is a folder                         
                        if (row["ows_FileRef"].ToString().EndsWith(docLibName))
                        {
                            //it's the folder we queried on.  Not sure why this is returned.
                        }
                        else
                        {
                            if (this.Recursive)
                                DownloadFolderItems(listName, docLibName + "/" + fr.FileName);
                        }
                    }
                }
            }
        }

        #endregion
    }
}
